<?php
/**
 * Copyright © 2015 FiloBlu. All rights reserved.
 */

namespace FiloBlu\Flow\Setup;

use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\Setup\ModuleContextInterface;
/**
 * @codeCoverageIgnore
 */
class UpgradeSchema implements UpgradeSchemaInterface
{
    /**
     * {@inheritdoc}
     */
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        
        $setup->startSetup();

        if(!$context->getVersion()) {
            
        }
        
        if (version_compare($context->getVersion(), '0.0.8') < 0) {
              $this->createFlowFromStockTable($setup, $context);
        }
        
        if (version_compare($context->getVersion(), '0.1.1') < 0) {
             $this->addImagesTable($setup, $context);
        }
        
        if (version_compare($context->getVersion(), '0.1.2') < 0) {
             $this->addUniqueConstraints($setup, $context);
        }

        $setup->endSetup();

    }
        
     public function createFlowFromStockTable(SchemaSetupInterface $setup, ModuleContextInterface $context)
        {
         
          /**
           * Create table 'flow_from_stock' if not exists
           */

          if (!$setup->tableExists('flow_from_stock')) {

              $table = $setup->getConnection()->newTable(
                  $setup->getTable('flow_from_stock')
              )
                  ->addColumn(
                        'sku',
                        \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        '40',
                        [],
                        'sku'
                  )
                  ->addColumn(
                        'magazzino',
                        \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        '30',
                        [],
                        'magazzino'
                  )
                  ->addColumn(
                        'fisico_disponibile',
                        \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                        '10',
                        [],
                        'qta disponibile in magazzino - aggiornamento in sovrascrittura'
                  )
                  ->addColumn(
                        'meta_id',
                        \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                        null,
                        ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                        'id'
                    )
                  ->addColumn(
                        'meta_processed',
                        \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                        '1',
                        [],
                        'meta_processed'
                    )
                  ->addColumn(
                        'meta_process_time',
                        \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                        '10',
                        [],
                        'meta_process_time'
                    )

                  ->setComment(
                      'FiloBlu Flow flow_from_stock'
                  );

              $setup->getConnection()->createTable($table);


          }

         
        }

        public function addImagesTable ($setup, $context) {

        /**
         * Create table 'filoblu_imageflow_from' if not exists
         */

        if (!$setup->tableExists('flow_imagesflow')) {

            $table = $setup->getConnection()->newTable(
                $setup->getTable('flow_imagesflow')
            )
                ->addColumn(
                    'id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                    null
                )
                ->addColumn(
                    'image_name',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    null
                )
                ->addColumn(
                    'image_status',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '30',
                    [],
                    null
                )
                ->addColumn(
                    'image_queue_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    null,
                    [],
                    null
                )
                ->addColumn(
                    'image_receive_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    null,
                    [],
                    null
                )
                ->addColumn(
                    'image_process_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    null,
                    [],
                    null
                )        
                ->addColumn(
                    'meta_file',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['nullable' => false],
                    null
                )                       
                ->addColumn(
                    'meta_processed',
                    \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                    "1",
                    ['nullable' => false, 'default' => '0'],
                    null
                )   
                ->addColumn(
                    'log',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    null
                );                    

                /*{{CedAddTableColumn}}}*/

            $setup->getConnection()->createTable($table);
            /*{{CedAddTable}}*/


        }
      }
      
      public function addUniqueConstraints(SchemaSetupInterface $setup, ModuleContextInterface $context)
        {
            try{
                $setup->getConnection()->query("ALTER TABLE `flow_inboundflow` ADD UNIQUE (`name`(255));");
                $setup->getConnection()->query("ALTER TABLE `flow_channels` ADD UNIQUE (`name`(40));");
            } catch(\Exception $e) {
                throw new \Exception("Error Processing Request - add unique constraints on `flow_inboundflow` and `flow_channels` " . $e->getMessage() , 1);
            }
        }

}
